@@ -34,9 +34,9 @@
 #include <X11/Xfuncs.h>
 
 #include <X11/X.h>
-#define	NEED_EVENTS
 #include <X11/keysym.h>
 #include <X11/Xproto.h>
+#include <X11/extensions/XKMformat.h>
 #include "misc.h"
 #include "inputstr.h"
 #include "dix.h"
